@@ -106,7 +106,13 @@ external caml_av_input_io_finalise : avio -> unit = "caml_av_input_io_finalise"
 let open_input_stream ?format ?opts ?seek read =
   let avio = ocaml_av_create_read_io 4096 ?seek read in
   let cleanup () = caml_av_input_io_finalise avio in
-  let input = ocaml_av_open_input_stream ?format ?opts avio in
+  let input =
+    try ocaml_av_open_input_stream ?format ?opts avio
+    with exn ->
+      let bt = Printexc.get_raw_backtrace () in
+      cleanup ();
+      Printexc.raise_with_backtrace exn bt
+  in
   Gc.finalise_last cleanup input;
   input
 
@@ -139,6 +145,13 @@ let mk_stream container index = { container; index; decoder = None }
 external get_codec_params : (_, 'm, _) stream -> 'm Avcodec.params
   = "ocaml_av_get_stream_codec_parameters"
 
+external get_avg_frame_rate : (_, video, _) stream -> Avutil.rational option
+  = "ocaml_av_get_stream_avg_frame_rate"
+
+external set_avg_frame_rate :
+  (_, video, _) stream -> Avutil.rational option -> unit
+  = "ocaml_av_set_stream_avg_frame_rate"
+
 external get_time_base : (_, _, _) stream -> Avutil.rational
   = "ocaml_av_get_stream_time_base"
 
@@ -151,19 +164,19 @@ external get_frame_size : (_, audio, _) stream -> int
 external get_pixel_aspect : (_, video, _) stream -> Avutil.rational option
   = "ocaml_av_get_stream_pixel_aspect"
 
-external _get_streams : input container -> media_type -> int list
+external _get_streams : _ container -> media_type -> int list
   = "ocaml_av_get_streams"
 
-let get_streams input media_type =
-  _get_streams input media_type
+let get_streams container media_type =
+  _get_streams container media_type
   |> List.rev_map (fun i ->
-         let s = mk_stream input i in
+         let s = mk_stream container i in
          (i, s, get_codec_params s))
 
-let get_audio_streams input = get_streams input MT_audio
-let get_video_streams input = get_streams input MT_video
-let get_subtitle_streams input = get_streams input MT_subtitle
-let get_data_streams input = get_streams input MT_data
+let get_audio_streams container = get_streams container MT_audio
+let get_video_streams container = get_streams container MT_video
+let get_subtitle_streams container = get_streams container MT_subtitle
+let get_data_streams container = get_streams container MT_data
 let set_decoder s decoder = s.decoder <- Some decoder
 
 external _find_best_stream : input container -> media_type -> int
@@ -279,7 +292,11 @@ let open_output_stream ?opts ?(interleaved = true) ?seek write format =
   let avio = ocaml_av_create_io 4096 None (Some write) (_seek_of_seek seek) in
   let cleanup () = caml_av_input_io_finalise avio in
   let output, unused =
-    ocaml_av_open_output_stream format avio interleaved (mk_opts_array opts)
+    try ocaml_av_open_output_stream format avio interleaved (mk_opts_array opts)
+    with exn ->
+      let bt = Printexc.get_raw_backtrace () in
+      cleanup ();
+      Printexc.raise_with_backtrace exn bt
   in
   filter_opts unused opts;
   Gc.finalise ocaml_av_cleanup_av output;
@@ -318,30 +335,20 @@ external new_audio_stream :
   _ container ->
   int ->
   [ `Encoder ] Avcodec.Audio.t ->
-  int ->
+  Channel_layout.t ->
   (string * string) array ->
   int * string array = "ocaml_av_new_audio_stream"
 
-let new_audio_stream ?opts ?channels ?channel_layout ~sample_rate ~sample_format
+let new_audio_stream ?opts ~channel_layout ~sample_rate ~sample_format
     ~time_base ~codec container =
   let opts =
-    mk_audio_opts ?opts ?channels ?channel_layout ~sample_rate ~sample_format
-      ~time_base ()
-  in
-  let channels =
-    match (channels, channel_layout) with
-      | Some n, _ -> n
-      | None, Some layout -> Avutil.Channel_layout.get_nb_channels layout
-      | None, None ->
-          raise
-            (Error
-               (`Failure
-                 "At least one of channels or channel_layout must be passed!"))
+    mk_audio_opts ?opts ~channel_layout ~sample_rate ~sample_format ~time_base
+      ()
   in
   let ret, unused =
     new_audio_stream container
       (Sample_format.get_id sample_format)
-      codec channels (mk_opts_array opts)
+      codec channel_layout (mk_opts_array opts)
   in
   filter_opts unused opts;
   mk_stream container ret
@@ -370,7 +377,9 @@ let new_video_stream ?opts ?frame_rate ?hardware_context ~pixel_format ~width
       (mk_opts_array opts)
   in
   filter_opts unused opts;
-  mk_stream container ret
+  let s = mk_stream container ret in
+  set_avg_frame_rate s frame_rate;
+  s
 
 external new_subtitle_stream :
   _ container ->
@@ -403,16 +412,10 @@ external write_packet :
   unit = "ocaml_av_write_stream_packet"
 
 external write_frame :
-  (output, 'media, [ `Frame ]) stream -> 'media frame -> unit
-  = "ocaml_av_write_stream_frame"
+  ?on_keyframe:(unit -> unit) ->
+  (output, 'media, [ `Frame ]) stream ->
+  'media frame ->
+  unit = "ocaml_av_write_stream_frame"
 
 external flush : output container -> unit = "ocaml_av_flush"
-external was_keyframe : (output, _, _) stream -> bool = "ocaml_av_was_keyframe"
-
-external write_audio_frame : output container -> audio frame -> unit
-  = "ocaml_av_write_audio_frame"
-
-external write_video_frame : output container -> video frame -> unit
-  = "ocaml_av_write_video_frame"
-
 external close : _ container -> unit = "ocaml_av_close"
