@@ -22,7 +22,9 @@
 #include <libavutil/audio_fifo.h>
 #include <libavutil/avstring.h>
 #include <libavutil/opt.h>
+#include <libavutil/parseutils.h>
 #include <libavutil/timestamp.h>
+#include <libavutil/mem.h>
 
 #include "av_stubs.h"
 #include "avcodec_stubs.h"
@@ -46,9 +48,6 @@ typedef struct {
 
   // input
   int got_frame;
-
-  // output
-  int was_keyframe;
 } stream_t;
 
 typedef struct av_t {
@@ -198,6 +197,44 @@ CAMLprim value ocaml_av_get_stream_codec_parameters(value _stream) {
   CAMLreturn(ans);
 }
 
+CAMLprim value ocaml_av_get_stream_avg_frame_rate(value _stream) {
+  CAMLparam1(_stream);
+  CAMLlocal3(ans, ret, _av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
+  int index = StreamIndex_val(_stream);
+  AVStream *st = av->format_context->streams[index];
+
+  if (!st->avg_frame_rate.num)
+    CAMLreturn(Val_none);
+
+  value_of_rational(&av->format_context->streams[index]->avg_frame_rate, &ans);
+
+  ret = caml_alloc_tuple(1);
+  Store_field(ret, 0, ans);
+
+  CAMLreturn(ret);
+}
+
+CAMLprim value ocaml_av_set_stream_avg_frame_rate(value _stream,
+                                                  value _avg_frame_rate) {
+  CAMLparam2(_stream, _avg_frame_rate);
+  CAMLlocal1(_av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
+  int index = StreamIndex_val(_stream);
+  AVStream *st = av->format_context->streams[index];
+
+  if (_avg_frame_rate == Val_none) {
+    st->avg_frame_rate = (AVRational){0, 1};
+    CAMLreturn(Val_unit);
+  }
+
+  st->avg_frame_rate = rational_of_value(Field(_avg_frame_rate, 0));
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_av_get_stream_time_base(value _stream) {
   CAMLparam1(_stream);
   CAMLlocal2(ans, _av);
@@ -360,8 +397,13 @@ static int ocaml_avio_read_callback(void *private, uint8_t *buf, int buf_size) {
   return Int_val(res);
 }
 
+#if LIBAVFORMAT_VERSION_MAJOR < 61
 static int ocaml_avio_write_callback(void *private, uint8_t *buf,
                                      int buf_size) {
+#else
+static int ocaml_avio_write_callback(void *private, const uint8_t *buf,
+                                     int buf_size) {
+#endif
   value buffer, res;
   avio_t *avio = (avio_t *)private;
   int ret;
@@ -484,7 +526,11 @@ CAMLprim value ocaml_av_create_io(value bufsize, value _read_cb,
   CAMLlocal1(ret);
 
   int (*read_cb)(void *opaque, uint8_t *buf, int buf_size) = NULL;
+#if LIBAVFORMAT_VERSION_MAJOR < 61
   int (*write_cb)(void *opaque, uint8_t *buf, int buf_size) = NULL;
+#else
+  int (*write_cb)(void *opaque, const uint8_t *buf, int buf_size) = NULL;
+#endif
   int64_t (*seek_cb)(void *opaque, int64_t offset, int whence) = NULL;
   int write_flag = 0;
   unsigned char *buffer;
@@ -599,7 +645,7 @@ CAMLprim value ocaml_av_find_input_format(value _short_name) {
   CAMLparam1(_short_name);
   CAMLlocal1(ret);
   char *short_name =
-      strndup(String_val(_short_name), caml_string_length(_short_name));
+      av_strndup(String_val(_short_name), caml_string_length(_short_name));
 
   if (!short_name)
     caml_raise_out_of_memory();
@@ -692,7 +738,7 @@ static av_t *open_input(char *url, avioformat_const AVInputFormat *format,
   caml_acquire_runtime_system();
 
   if (err < 0) {
-    avformat_free_context(av->format_context);
+    avformat_close_input(&av->format_context);
     if (av->interrupt_cb != Val_none) {
       caml_remove_generational_global_root(&av->interrupt_cb);
       av->interrupt_cb = Val_none;
@@ -731,7 +777,7 @@ CAMLprim value ocaml_av_open_input(value _url, value _format, value _interrupt,
   }
 
   if (ulen > 0)
-    url = strndup(String_val(_url), ulen);
+    url = av_strndup(String_val(_url), ulen);
 
   if (_format != Val_none)
     format = InputFormat_val(Some_val(_format));
@@ -1306,13 +1352,13 @@ CAMLprim value ocaml_av_output_format_guess(value _short_name, value _filename,
 
   if (caml_string_length(_short_name) > 0) {
     short_name =
-        strndup(String_val(_short_name), caml_string_length(_short_name));
+        av_strndup(String_val(_short_name), caml_string_length(_short_name));
     if (!short_name)
       caml_raise_out_of_memory();
   };
 
   if (caml_string_length(_filename) > 0) {
-    filename = strndup(String_val(_filename), caml_string_length(_filename));
+    filename = av_strndup(String_val(_filename), caml_string_length(_filename));
     if (!filename) {
       if (short_name)
         free(short_name);
@@ -1321,7 +1367,7 @@ CAMLprim value ocaml_av_output_format_guess(value _short_name, value _filename,
   }
 
   if (caml_string_length(_mime) > 0) {
-    mime = strndup(String_val(_mime), caml_string_length(_mime));
+    mime = av_strndup(String_val(_mime), caml_string_length(_mime));
     if (!mime) {
       if (short_name)
         free(short_name);
@@ -1508,7 +1554,7 @@ CAMLprim value ocaml_av_open_output(value _interrupt, value _format,
   CAMLparam3(_interrupt, _filename, _opts);
   CAMLlocal3(ans, ret, unused);
   char *filename =
-      strndup(String_val(_filename), caml_string_length(_filename));
+      av_strndup(String_val(_filename), caml_string_length(_filename));
   avioformat_const AVOutputFormat *format = NULL;
   AVDictionary *options = NULL;
   char *key, *val;
@@ -1720,7 +1766,7 @@ static stream_t *new_stream(av_t *av, const AVCodec *codec) {
 
   AVStream *avstream = avformat_new_stream(av->format_context, codec);
   if (!avstream) {
-    free(stream);
+    free_stream(stream);
     caml_raise_out_of_memory();
   }
 
@@ -1776,18 +1822,21 @@ static void init_stream_encoder(AVBufferRef *device_ctx, AVBufferRef *frame_ctx,
 }
 
 static stream_t *new_audio_stream(av_t *av, enum AVSampleFormat sample_fmt,
-                                  int channels, const AVCodec *codec,
+                                  AVChannelLayout *channel_layout,
+                                  const AVCodec *codec,
                                   AVDictionary **options) {
   stream_t *stream = new_stream(av, codec);
+  int ret;
 
   AVCodecContext *enc_ctx = stream->codec_context;
 
   enc_ctx->sample_fmt = sample_fmt;
-  enc_ctx->channels = channels;
-// Detect new API
-#ifdef AV_CHANNEL_LAYOUT_MONO
-  av_channel_layout_default(&enc_ctx->ch_layout, channels);
-#endif
+  ret = av_channel_layout_copy(&enc_ctx->ch_layout, channel_layout);
+
+  if (ret < 0) {
+    free_stream(stream);
+    ocaml_avutil_raise_error(ret);
+  }
 
   init_stream_encoder(NULL, NULL, av, stream, options);
 
@@ -1809,9 +1858,9 @@ CAMLprim value ocaml_av_initialize_stream_copy(value _av, value _stream_index,
   av_t *av = Av_val(_av);
 
   AVStream *avstream = av->format_context->streams[Int_val(_stream_index)];
+  AVCodecParameters *params = CodecParameters_val(_params);
 
-  int err =
-      avcodec_parameters_copy(avstream->codecpar, CodecParameters_val(_params));
+  int err = avcodec_parameters_copy(avstream->codecpar, params);
   if (err < 0)
     ocaml_avutil_raise_error(err);
 
@@ -1821,7 +1870,7 @@ CAMLprim value ocaml_av_initialize_stream_copy(value _av, value _stream_index,
 }
 
 CAMLprim value ocaml_av_new_audio_stream(value _av, value _sample_fmt,
-                                         value _codec, value _channels,
+                                         value _codec, value _channel_layout,
                                          value _opts) {
   CAMLparam2(_av, _opts);
   CAMLlocal2(ans, unused);
@@ -1843,8 +1892,9 @@ CAMLprim value ocaml_av_new_audio_stream(value _av, value _sample_fmt,
     }
   }
 
-  stream_t *stream = new_audio_stream(Av_val(_av), Int_val(_sample_fmt),
-                                      Int_val(_channels), codec, &options);
+  stream_t *stream =
+      new_audio_stream(Av_val(_av), Int_val(_sample_fmt),
+                       AVChannelLayout_val(_channel_layout), codec, &options);
 
   // Return unused keys
   count = av_dict_count(options);
@@ -2041,7 +2091,6 @@ CAMLprim value ocaml_av_write_stream_packet(value _stream, value _time_base,
                        avstream->time_base);
 
   ret = av->write_frame(av->format_context, packet);
-  av->streams[stream_index]->was_keyframe = packet->flags & AV_PKT_FLAG_KEY;
 
   caml_acquire_runtime_system();
 
@@ -2052,7 +2101,7 @@ CAMLprim value ocaml_av_write_stream_packet(value _stream, value _time_base,
 }
 
 static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
-                        AVFrame *frame) {
+                        value _on_keyframe, AVFrame *frame) {
   AVStream *avstream = av->format_context->streams[stream_index];
   AVFrame *hw_frame = NULL;
   int ret;
@@ -2142,8 +2191,6 @@ static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
     ocaml_avutil_raise_error(ret);
   }
 
-  int was_keyframe = 0;
-
   // read all the available output packets (in general there may be any number
   // of them
   while (ret >= 0) {
@@ -2152,8 +2199,16 @@ static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
     if (ret < 0)
       break;
 
-    if (packet->flags & AV_PKT_FLAG_KEY)
-      was_keyframe = 1;
+    if (packet->flags & AV_PKT_FLAG_KEY && _on_keyframe != Val_none) {
+      caml_acquire_runtime_system();
+
+      if (ret < 0)
+        ocaml_avutil_raise_error(ret);
+
+      caml_callback(Field(_on_keyframe, 0), Val_unit);
+
+      caml_release_runtime_system();
+    }
 
     packet->stream_index = stream_index;
     packet->pos = -1;
@@ -2166,8 +2221,6 @@ static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
     av_frame_free(&hw_frame);
   av_packet_free(&packet);
 
-  av->streams[stream_index]->was_keyframe = was_keyframe;
-
   caml_acquire_runtime_system();
 
   if (!frame && ret == AVERROR_EOF)
@@ -2177,7 +2230,8 @@ static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
     ocaml_avutil_raise_error(ret);
 }
 
-static void write_audio_frame(av_t *av, int stream_index, AVFrame *frame) {
+static void write_audio_frame(av_t *av, int stream_index, value _on_keyframe,
+                              AVFrame *frame) {
   int err, frame_size;
 
   if (av->format_context->nb_streams < stream_index)
@@ -2190,10 +2244,11 @@ static void write_audio_frame(av_t *av, int stream_index, AVFrame *frame) {
 
   AVCodecContext *enc_ctx = stream->codec_context;
 
-  write_frame(av, stream_index, enc_ctx, frame);
+  write_frame(av, stream_index, enc_ctx, _on_keyframe, frame);
 }
 
-static void write_video_frame(av_t *av, int stream_index, AVFrame *frame) {
+static void write_video_frame(av_t *av, int stream_index, value _on_keyframe,
+                              AVFrame *frame) {
   if (av->format_context->nb_streams < stream_index)
     Fail("Stream index not found!");
 
@@ -2207,7 +2262,7 @@ static void write_video_frame(av_t *av, int stream_index, AVFrame *frame) {
 
   AVCodecContext *enc_ctx = stream->codec_context;
 
-  write_frame(av, stream_index, enc_ctx, frame);
+  write_frame(av, stream_index, enc_ctx, _on_keyframe, frame);
 }
 
 static void write_subtitle_frame(av_t *av, int stream_index,
@@ -2267,8 +2322,9 @@ static void write_subtitle_frame(av_t *av, int stream_index,
     ocaml_avutil_raise_error(err);
 }
 
-CAMLprim value ocaml_av_write_stream_frame(value _stream, value _frame) {
-  CAMLparam2(_stream, _frame);
+CAMLprim value ocaml_av_write_stream_frame(value _on_keyframe, value _stream,
+                                           value _frame) {
+  CAMLparam3(_on_keyframe, _stream, _frame);
   CAMLlocal1(_av);
   _av = Field(_stream, 0);
   av_t *av = Av_val(_av);
@@ -2280,9 +2336,9 @@ CAMLprim value ocaml_av_write_stream_frame(value _stream, value _frame) {
   enum AVMediaType type = av->streams[index]->codec_context->codec_type;
 
   if (type == AVMEDIA_TYPE_AUDIO) {
-    write_audio_frame(av, index, Frame_val(_frame));
+    write_audio_frame(av, index, _on_keyframe, Frame_val(_frame));
   } else if (type == AVMEDIA_TYPE_VIDEO) {
-    write_video_frame(av, index, Frame_val(_frame));
+    write_video_frame(av, index, _on_keyframe, Frame_val(_frame));
   } else if (type == AVMEDIA_TYPE_SUBTITLE) {
     write_subtitle_frame(av, index, Subtitle_val(_frame));
   }
@@ -2310,39 +2366,6 @@ CAMLprim value ocaml_av_flush(value _av) {
   CAMLreturn(Val_unit);
 }
 
-CAMLprim value ocaml_av_was_keyframe(value _stream) {
-  CAMLparam1(_stream);
-  CAMLlocal1(_av);
-  _av = Field(_stream, 0);
-  av_t *av = Av_val(_av);
-  int index = StreamIndex_val(_stream);
-
-  if (!av->streams)
-    Fail("Invalid input: no streams provided");
-
-  CAMLreturn(Val_bool(av->streams[index]->was_keyframe));
-}
-
-CAMLprim value ocaml_av_write_audio_frame(value _av, value _frame) {
-  CAMLparam2(_av, _frame);
-  av_t *av = Av_val(_av);
-  AVFrame *frame = Frame_val(_frame);
-
-  write_audio_frame(av, 0, frame);
-
-  CAMLreturn(Val_unit);
-}
-
-CAMLprim value ocaml_av_write_video_frame(value _av, value _frame) {
-  CAMLparam2(_av, _frame);
-  av_t *av = Av_val(_av);
-  AVFrame *frame = Frame_val(_frame);
-
-  write_video_frame(av, 0, frame);
-
-  CAMLreturn(Val_unit);
-}
-
 CAMLprim value ocaml_av_close(value _av) {
   CAMLparam1(_av);
   av_t *av = Av_val(_av);
@@ -2358,9 +2381,9 @@ CAMLprim value ocaml_av_close(value _av) {
         continue;
 
       if (enc_ctx->codec_type == AVMEDIA_TYPE_AUDIO) {
-        write_audio_frame(av, i, NULL);
+        write_audio_frame(av, i, Val_none, NULL);
       } else if (enc_ctx->codec_type == AVMEDIA_TYPE_VIDEO) {
-        write_video_frame(av, i, NULL);
+        write_video_frame(av, i, Val_none, NULL);
       }
     }
 
@@ -2531,6 +2554,7 @@ CAMLprim value ocaml_av_stream_bitrate(value _stream) {
   _av = Field(_stream, 0);
   av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
+  AVCPBProperties *props = NULL;
 
   if (!av->format_context || !av->format_context->streams)
     CAMLreturn(Val_none);
@@ -2540,19 +2564,30 @@ CAMLprim value ocaml_av_stream_bitrate(value _stream) {
   if (!stream)
     CAMLreturn(Val_none);
 
-  AVCPBProperties *props = (AVCPBProperties *)av_stream_get_side_data(
-      stream, AV_PKT_DATA_CPB_PROPERTIES, NULL);
+  if (stream->codecpar->bit_rate) {
+    ans = caml_alloc_tuple(1);
+    Store_field(ans, 0, Val_int(stream->codecpar->bit_rate));
+    CAMLreturn(ans);
+  }
 
-  if (!stream->codecpar->bit_rate && !props)
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(60, 15, 100)
+  const AVPacketSideData *side_data = av_packet_side_data_get(
+      stream->codecpar->coded_side_data, stream->codecpar->nb_coded_side_data,
+      AV_PKT_DATA_CPB_PROPERTIES);
+
+  if (!side_data)
+    CAMLreturn(Val_none);
+
+  props = (AVCPBProperties *)side_data->data;
+#else
+  props = (AVCPBProperties *)av_stream_get_side_data(
+      stream, AV_PKT_DATA_CPB_PROPERTIES, NULL);
+#endif
+
+  if (!props)
     CAMLreturn(Val_none);
 
   ans = caml_alloc_tuple(1);
-
-  if (stream->codecpar->bit_rate) {
-    Store_field(ans, 0, Val_int(stream->codecpar->bit_rate));
-  } else if (props) {
-    Store_field(ans, 0, Val_int(props->max_bitrate));
-  }
-
+  Store_field(ans, 0, Val_int(props->max_bitrate));
   CAMLreturn(ans);
 }
