@@ -329,7 +329,11 @@ module Audio = struct
   let find_best_channel_layout codec default =
     try
       let channel_layouts = get_supported_channel_layouts codec in
-      if List.mem default channel_layouts then default
+      if
+        List.exists
+          (fun layout -> Avutil.Channel_layout.compare layout default)
+          channel_layouts
+      then default
       else (match channel_layouts with h :: _ -> h | [] -> default)
     with Not_found -> default
 
@@ -384,31 +388,21 @@ module Audio = struct
   external create_encoder :
     int ->
     [ `Encoder ] t ->
-    int ->
+    Channel_layout.t ->
     (string * string) array ->
     audio encoder * string array = "ocaml_avcodec_create_audio_encoder"
 
-  let create_encoder ?opts ?channels ?channel_layout ~sample_rate ~sample_format
+  let create_encoder ?opts ~channel_layout ~sample_rate ~sample_format
       ~time_base codec =
     let opts = opts_default opts in
     let _opts =
-      mk_audio_opts ~opts ?channels ?channel_layout ~sample_rate ~sample_format
-        ~time_base ()
-    in
-    let channels =
-      match (channels, channel_layout) with
-        | Some n, _ -> n
-        | None, Some layout -> Avutil.Channel_layout.get_nb_channels layout
-        | None, None ->
-            raise
-              (Error
-                 (`Failure
-                   "At least one of channels or channel_layout must be passed!"))
+      mk_audio_opts ~opts ~channel_layout ~sample_rate ~sample_format ~time_base
+        ()
     in
     let encoder, unused =
       create_encoder
         (Sample_format.get_id sample_format)
-        codec channels (mk_opts_array _opts)
+        codec channel_layout (mk_opts_array _opts)
     in
     filter_opts unused opts;
     encoder
