@@ -14,6 +14,7 @@
 #include <libavfilter/avfilter.h>
 #include <libavfilter/buffersink.h>
 #include <libavfilter/buffersrc.h>
+#include <libavutil/mem.h>
 
 #define AvFilterContext_val(v) (*(AVFilterContext **)Data_abstract_val(v))
 
@@ -178,12 +179,12 @@ CAMLprim value ocaml_avfilter_create_filter(value _args, value _instance_name,
     caml_raise_not_found();
 
   name =
-      strndup(String_val(_instance_name), caml_string_length(_instance_name));
+      av_strndup(String_val(_instance_name), caml_string_length(_instance_name));
   if (!name)
     caml_raise_out_of_memory();
 
   if (_args != Val_none) {
-    args = strndup(String_val(Some_val(_args)),
+    args = av_strndup(String_val(Some_val(_args)),
                    caml_string_length(Some_val(_args)));
 
     if (!args) {
@@ -309,7 +310,7 @@ CAMLprim value ocaml_avfilter_parse(value _inputs, value _outputs,
     append_avfilter_in_out(&outputs, name, filter_ctx, idx);
   }
 
-  filters = strndup(String_val(_filters), caml_string_length(_filters));
+  filters = av_strndup(String_val(_filters), caml_string_length(_filters));
 
   if (!filters) {
     if (inputs)
@@ -465,11 +466,14 @@ CAMLprim value ocaml_avfilter_buffersink_get_channels(value _src) {
 
 CAMLprim value ocaml_avfilter_buffersink_get_channel_layout(value _src) {
   CAMLparam0();
+  CAMLlocal1(ret);
   AVFilterContext *filter_ctx = AvFilterContext_val(_src);
+  AVChannelLayout *channel_layout =
+      caml_avutil_alloc_custom_channel_layout(&ret);
 
-  uint64_t layout = av_buffersink_get_channel_layout(filter_ctx);
+  int err = av_buffersink_get_ch_layout(filter_ctx, channel_layout);
 
-  CAMLreturn(Val_ChannelLayout(layout));
+  CAMLreturn(ret);
 }
 
 CAMLprim value ocaml_avfilter_buffersink_get_sample_rate(value _src) {
