@@ -16,14 +16,13 @@ let () =
   let audio_input, oass =
     Av.find_best_audio_stream src |> fun (i, audio_input, params) ->
     let channel_layout = Avcodec.Audio.get_channel_layout params in
-    let channels = Avcodec.Audio.get_nb_channels params in
     let sample_format = Avcodec.Audio.get_sample_format params in
     let sample_rate = Avcodec.Audio.get_sample_rate params in
     let time_base = { Avutil.num = 1; den = sample_rate } in
     ( audio_input,
       ( i,
-        Av.new_audio_stream ~channels ~channel_layout ~sample_format
-          ~sample_rate ~time_base ~codec:audio_codec dst ) )
+        Av.new_audio_stream ~channel_layout ~sample_format ~sample_rate
+          ~time_base ~codec:audio_codec dst ) )
   in
 
   let frame_rate = { Avutil.num = 25; den = 1 } in
